#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Budy
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Budy.
#
# Hive Budy is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Budy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Budy. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from . import base


class Store(base.BudyBase):
    """
    Represents a physical store entity that can be used
    for the operations like instore pick-up, affecting
    the checkout process.
    """

    name = appier.field(index=True, default=True)

    address = appier.field(type=appier.reference("Address", name="id"), eager=True)

    currency_code = appier.field(index=True)

    checkout_mode = appier.field(index=True)

    restrict_mode = appier.field(type=bool, index=True, initial=True)

    @classmethod
    def validate(cls):
        return super(Store, cls).validate() + [
            appier.not_null("name"),
            appier.not_empty("name"),
            appier.string_eq("currency_code", 3),
        ]

    @classmethod
    def list_names(cls):
        return ["id", "name", "currency_code", "checkout_mode"]

    @property
    def is_restricted(self):
        return self.restrict_mode
